# Rules.pm - Rules file for generator (IPTables Firewall Configuration).
# Created by James A. Pattie.  Copyright (c) 2001-2005, Xperience, Inc.

package PCXFireWall::Rules;

use strict;
use vars qw ($AUTOLOAD @ISA @EXPORT $VERSION);

require Exporter;
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = "2.6";

# ****************** User Configuration Starts here ***************************
#  Only edit between "Define your rules here" and "Stop Defining your rules"
# *****************************************************************************

# void startingRules(configObj, interfaceObj, filterObj, natObj, mangleObj, proc, modulesObj, networkObj)
#   Defines the rules/actions that need to be done when the firewall is starting up.
sub startingRules
{
  my $self = shift;
  my %args = ( configObj => undef, interfaceObj => undef, filterObj => undef, natObj => undef, mangleObj => undef, proc => undef, modulesObj => undef, networkObj => undef, @_ );
  my $config = $args{configObj};
  my $filter = $args{filterObj};
  my $nat = $args{natObj};
  my $mangle = $args{mangleObj};
  my $interfaces = $args{interfaceObj};
  my $proc = $args{proc};
  my $modulesObj = $args{modulesObj};
  my $networkObj = $args{networkObj};

  # -------------------- Define your rules here -------------------------------

  # specify the netfilter modules needed.
  $modulesObj->kernelType(type => "modular");  # set to monolithic if you don't want this.
  $modulesObj->register(module => "ip_conntrack");
  $modulesObj->register(module => "iptable_nat");
  $modulesObj->register(module => "iptable_mangle");
  $modulesObj->register(module => "ipt_limit");
  $modulesObj->register(module => "ipt_mark");
  $modulesObj->register(module => "ipt_MARK");
  $modulesObj->register(module => "ipt_MASQUERADE");
  $modulesObj->register(module => "ipt_REDIRECT");
  $modulesObj->register(module => "ipt_REJECT");
  $modulesObj->register(module => "ipt_state");
  $modulesObj->register(module => "ipt_tos");
  $modulesObj->register(module => "ipt_TOS");
  $modulesObj->register(module => "ipt_unclean");
  $modulesObj->register(module => "ipt_multiport");
  $modulesObj->register(module => "ip_conntrack_ftp");
  $modulesObj->register(module => "ip_conntrack_irc");
  $modulesObj->register(module => "ip_nat_ftp");
  $modulesObj->register(module => "ip_nat_irc");

  # change log-prefix value
  $config->logPrefix("PCX FW");  # This is the value used if you don't specify the log-prefix value.

  # set default post-comment to be a \n so we have some spacing.
  $config->postComment("\n");
  $config->preComment("");

  # Uncomment the following line if you want to use 'ip' instead of 'ifconfig'.
  #$config->networkCommand("ip");

  # turn on synCookie and Martian logging support
  $proc->sysNetIPv4(file => "tcp_syncookies", value => "1");
  $proc->sysNetIPv4Conf(file => "log_martians", value => "1", iface => "all");

  # turn on/off ip forwarding, dynamic IP Addresses, icmp redirects, anti-spoofing blocks and source routing
  $proc->sysNetIPv4(file => "ip_forward", value => "1", module => "Nat", when => "Post", required => "true");
  $proc->sysNetIPv4(file => "ip_dynaddr", value => "1", module => "Nat");
  $proc->sysNetIPv4Conf(file => "rp_filter", value => "1", module => "Nat", iface => "all");
  #$proc->sysNetIPv4Conf(file => "rp_filter", value => "0", module => "Nat", iface => "eth0");  # turn off anti-spoofing on eth0
  $proc->sysNetIPv4Conf(file => "accept_source_route", value => "0", module => "Nat", iface => "all");
  $proc->sysNetIPv4Conf(file => "accept_redirects", value => "0", module => "Nat", iface => "all");
  $proc->sysNetIPv4(file => "icmp_echo_ignore_broadcasts", value => "1", module => "Nat");

  my $logLevel = "warn";

  my $externalInterface = "eth0:1";  # I'm defining these as perl variables so that I am not hardcoding the string in the method calls.
  my $internalInterface = "eth1";
  my $internalNetwork = "\${NETWORK_0}";


  my $dhcpServerIP = "\${NETWORK_1}";  # Replace with your ISP's DHCP server IP if
                                 # you have one (Cable modems, DSL, etc.)

  $networkObj->register(index => "0", type => "network", host => "192.168.0.0/24", comment => "Internal Network");
  $networkObj->register(index => "1", type => "network", host => "1.1.1.1", comment => "DHCP Server");

  # create the interface definitions
  $interfaces->defineInterface(name => "lo", location => "firewall", IP => "127.0.0.1", alias => "loopback");  # loopback interface
  $interfaces->defineInterface(name => "$externalInterface", type => "dynamic", alias => "external");  # external interface, using default value for location.
  $interfaces->defineInterface(name => "$internalInterface", location => "internal", IP => "192.168.0.1", alias => "internal");

  $interfaces->defineLocation(location => "vpn");
  $interfaces->defineInterface(name => "eth2", location => "vpn", IP => "192.168.0.2", alias => "vpn");

  # set chain policies
  $filter->policy(chain => "FORWARD", policy => "DROP");
  $filter->policy(chain => "INPUT", policy => "DROP");
  $filter->policy(chain => "OUTPUT", policy => "ACCEPT");
  $mangle->policy(chain => "OUTPUT", policy => "ACCEPT");
  $mangle->policy(chain => "PREROUTING", policy => "ACCEPT");
  $mangle->policy(chain => "INPUT", policy => "ACCEPT");
  $mangle->policy(chain => "FORWARD", policy => "ACCEPT");
  $mangle->policy(chain => "POSTROUTING", policy => "ACCEPT");
  $nat->policy(chain => "PREROUTING", policy => "ACCEPT");
  $nat->policy(chain => "POSTROUTING", policy => "ACCEPT");
  $nat->policy(chain => "OUTPUT", policy => "ACCEPT");

  # flush and delete all chains
  $filter->deleteChain(name => "All");
  $mangle->deleteChain(name => "All");
  $nat->deleteChain(name => "All");

  # setup chains to log then drop packets.
  $filter->createChain(name => "Tainted");
  $nat->createChain(name => "Tainted");
  $nat->createChain(name => "Reserved");

  $filter->log(chain => "Tainted", ruleMode => "setup", "log-limit" => 1, "log-limit-rate" => "20/m", "log-prefix" => "Tainted Packet: ", "log-level" => "$logLevel");
  $filter->drop(chain => "Tainted", ruleMode => "setup",);

  $nat->log(chain => "Tainted", ruleMode => "setup", "log-limit" => 1, "log-limit-rate" => "20/m", "log-prefix" => "Tainted Packet: ", "log-level" => "$logLevel");
  $nat->drop(chain => "Tainted", ruleMode => "setup",);

  $nat->log(chain => "Reserved", ruleMode => "setup", "log-limit" => 1, "log-limit-rate" => "20/m", "log-prefix" => "Reserved Packet: ", "log-level" => "$logLevel");
  $nat->drop(chain => "Reserved", ruleMode => "setup",);

  # handle unclean packets coming in.
  $filter->jump(chain => "INPUT", dchain => "Tainted", i => $externalInterface, unclean => 1);
  $nat->jump(chain => "PREROUTING", dchain => "Tainted", i => $externalInterface, unclean => 1);

  # block reserved/private addresses coming in the external interface.
  for my $i ("10.0.0.0/8", "172.16.0.0/12", "224.0.0.0/3", "0.0.0.0/8", "127.0.0.0/8", "169.254.0.0/16", "192.168.0.0/16")
  {
    $nat->jump(chain => "PREROUTING", dchain => "Reserved", i => $externalInterface, s => "$i", preComment => "# Block Reserved/Private Addresses coming in the external interface.\n");
  }

  # drop any smb traffic touching the box.
  $nat->drop(chain => "PREROUTING", p => "udp", sport => "138", dport => "138", preComment => "# Drop any smb traffic touching this box.  Don't log it.\n");

  # drop any bootp traffic touching the box that isn't from the ISP.
  #$nat->drop(chain => "PREROUTING", p => "udp", sport => "67", dport => "68", preComment => "# Drop any bootp traffic touching this box.  Don't log it.\n");

  # uncomment the previous line if you don't have a DHCP server and comment out
  # the following two lines.  Make sure the dhcpServerIP variable has the right
  # value set rather than 1.1.1.1
  $nat->drop(chain => "PREROUTING", p => "udp", sport => "67", dport => "68", s => "! $dhcpServerIP", preComment => "# Drop any bootp traffic touching this box.  Don't log it.\n");
  $filter->accept(chain => "INPUT", p => "udp", sport => "67", dport => "68", s => "$dhcpServerIP", preComment => "# Accept any bootp traffic from your ISP's DHCP Server.\n");

  # define traffic allowed into/out the box.

  $filter->reject(chain => "INPUT", p => "tcp", dport => "113", d => "inInterfaceIP", i => $externalInterface, "reject-with" => "tcp-reset", preComment => "# Make sure ident's don't just hang on us.\n", log => 1, "log-limit" => 1, "log-limit-rate" => "30/m", "log-level" => "$logLevel");
  $filter->accept(chain => "FORWARD", i => $internalInterface, s => $internalNetwork, preComment => "# Allow all internal traffic to go out.\n");
  $filter->accept(chain => "FORWARD", state => 1, states => "ESTABLISHED,RELATED", preComment => "# Accept all traffic that is already established and/or related.\n");

  $filter->accept(chain => "INPUT", p => "tcp", dport => "22", preComment => "# Accept any ssh traffic to the firewall.\n");
  $filter->accept(chain => "INPUT", p => "tcp", dport => "1000:1023", preComment => "# Accept any ssh forwarded session traffic to the firewall.\n");
  $filter->accept(chain => "INPUT", p => "icmp", limit => 1, "limit-rate" => "30/m", preComment => "# Allow all ICMP traffic to the firewall at the rate of 1 every 2 seconds.\n");
  # or you can just accept destination-unreachable, time-exceeded messages and drop everything else.
  $filter->accept(chain => "INPUT", p => "icmp", "icmp-type" => "destination-unreachable", preComment => "# Accept destination-unreachable icmp traffic.\n");
  $filter->accept(chain => "INPUT", p => "icmp", "icmp-type" => "time-exceeded", preComment => "# Accept time-exceeded icmp traffic.\n");

  # Define the next 2 rules if you are running DNS on the firewall itself.
  # Make sure you have the latest 'safe' version of bind running!
  $filter->accept(chain => "INPUT", p => "udp", dport => "53", preComment => "# Allow DNS traffic in.\n");
  $filter->accept(chain => "INPUT", p => "tcp", dport => "53", preComment => "# Allow DNS traffic in.\n");

  # This rule allows SMTP to the firewall (outside interface only).  Comment out if you are not running smtp on the firewall.
  $filter->accept(chain => "INPUT", p => "tcp", dport => "25", i => $externalInterface, preComment => "# Accept incoming SMTP connections from outside.\n");

  # allow all incoming traffic that is originating from the box itself.
  $filter->accept(chain => "INPUT", i => "lo", preComment => "# Accept all Locally Generated Traffic.\n");

  $filter->accept(chain => "INPUT", p => "tcp", dport => "3128", i => $internalInterface, s => $internalNetwork, preComment => "# Allow Squid traffic in from the internal network.\n");

  $filter->accept(chain => "INPUT", state => 1, states => "ESTABLISHED,RELATED");

  # Do any NAT necessary (choose snat or masquerade based upon your external interface's static or dynamic status)
  #$nat->snat(chain => "POSTROUTING", o => $externalInterface, "to-source" => "outInterfaceIP", preComment => "# SNAT any traffic going out eth0 to come from this box.\n\n");
  $nat->masquerade(chain => "POSTROUTING", o => $externalInterface, preComment => "# MASQUERADE all traffic leaving the firewall.\n\n");  # use this for dynamic interfaces.

  # any destination nat (DNAT), REDIRECTION and SNAT would be done here.

  # These 2 go together as a sample of making web traffic be completely transparent regardless of where you are coming from.
  #$nat->dnat(chain => "PREROUTING", dport => 80, p => "tcp", d => "192.168.0.1", "to-destination" => "192.168.0.100:80", preComment => "# DNAT web traffic to internal Web Server.\n"); # specify :80 on the destination is not needed.
  #$nat->snat(chain => "POSTROUTING", dport => 80, p => "tcp", d => "192.168.0.100", s => $internalNetwork, "to-source" => "192.168.0.1", preComment => "# Make any internal traffic being redirected to the internal web server\n# appear to come from the firewall so it gets properly DeNATed, etc.\n");

  # To redirect or allow SSH traffic, you not only have to allow port 22, but also ports 1000:1023 (at least for openSSH 2.x)

  # This is the beginnings of a transparent Squid Cache.  :)
  $nat->redirect(chain => "PREROUTING", dport => 80, p => "tcp", i => $internalInterface, s => $internalNetwork, "to-ports" => "3128", preComment => "# Redirect all outgoing web traffic to the Squid Cache.\n");

  # Do some simple protections for everyone involved (syn-flood, furtive port scanning, ping of death)
  # If you are doing stuff in the NAT module, then define any PREROUTING chain rules before these statements, else your traffic may match here and
  # things won't do what you expect.  :)
  $nat->accept(chain => "PREROUTING", i => $externalInterface, p => "tcp", syn => 1, limit => 1, "limit-rate" => "1/s", preComment => "# Syn-Flood Protect Us.\n");
  $filter->accept(chain => "FORWARD", o => $externalInterface, p => "tcp", syn => 1, limit => 1, "limit-rate" => "1/s", preComment => "# Syn-Flood Protect Them.\n");
  $filter->accept(chain => "OUTPUT", o => $externalInterface, p => "tcp", syn => 1, limit => 1, "limit-rate" => "1/s", preComment => "# Syn-Flood Protect Them.\n");

  $nat->accept(chain => "PREROUTING", i => $externalInterface, p => "tcp", "tcp-flags" => "SYN,ACK,FIN,RST RST", limit => 1, "limit-rate" => "1/s", preComment => "# Furtive Port Scan Protect Us.\n");
  $filter->accept(chain => "FORWARD", o => $externalInterface, p => "tcp", "tcp-flags" => "SYN,ACK,FIN,RST RST", limit => 1, "limit-rate" => "1/s", preComment => "# Furtive Port Scan Protect Them.\n");
  $filter->accept(chain => "OUTPUT", o => $externalInterface, p => "tcp", "tcp-flags" => "SYN,ACK,FIN,RST RST", limit => 1, "limit-rate" => "1/s", preComment => "# Furtive Port Scan Protect Them.\n");

  $nat->accept(chain => "PREROUTING", i => $externalInterface, p => "icmp", "icmp-type" => "echo-request", limit => 1, "limit-rate" => "1/s", preComment => "# Ping Of Death Protect Us.\n");
  $filter->accept(chain => "FORWARD", o => $externalInterface, p => "icmp", "icmp-type" => "echo-request", limit => 1, "limit-rate" => "1/s", preComment => "# Ping Of Death Protect Them.\n");
  $filter->accept(chain => "OUTPUT", o => $externalInterface, p => "icmp", "icmp-type" => "echo-request", limit => 1, "limit-rate" => "1/s", preComment => "# Ping Of Death Protect Them.\n");


  # Log the packets that are about to be dropped.  Don't define any INPUT or FORWARD chains after this point in Filter.
  $filter->log(chain => "FORWARD", "log-prefix" => "Default DROP - FORWARD chain:", limit => 1, "limit-rate" => "30/m", preComment => "# LOG all packets that are about to be dropped because of the POLICY.\n", "log-level" => "$logLevel");
  $filter->log(chain => "INPUT", "log-prefix" => "Default DROP - INPUT chain:", limit => 1, "limit-rate" => "30/m", preComment => "# LOG all packets that are about to be dropped because of the POLICY.\n", "log-level" => "$logLevel");

  # do some Mangling of packets.

  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 22);  # using default value of tosValue = 16.
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 21);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 23);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 25);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 53);
  $mangle->tos(chain => "OUTPUT", p => "udp", dport => 53);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 119);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 110);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 143);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 6667, preComment => "# IRC\n");
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 20, tosValue => 8);
  $mangle->tos(chain => "OUTPUT", p => "tcp", dport => 80, tosValue => 8);

  $mangle->tos(chain => "PREROUTING", p => "tcp", dport => 22);  # using default value of tosValue = 16.

  # -------------------- Stop Defining your rules -----------------------------
}

# void stoppingRules(configObj, interfaceObj, filterObj, natObj, mangleObj, proc, networkObj)
#   Defines the rules/actions that need to be done when the firewall is shutting down.
sub stoppingRules
{
  my $self = shift;
  my %args = ( configObj => undef, interfaceObj => undef, filterObj => undef, natObj => undef, mangleObj => undef, proc => undef, networkObj => undef, @_ );
  my $config = $args{configObj};
  my $filter = $args{filterObj};
  my $nat = $args{natObj};
  my $mangle = $args{mangleObj};
  my $interfaces = $args{interfaceObj};
  my $proc = $args{proc};
  my $networkObj = $args{networkObj};

  # -------------------- Define your rules here -------------------------------

  # change log-prefix value
  $config->logPrefix("PCX FW");  # This is the value used if you don't specify the log-prefix value.

  # set default post-comment to be a \n so we have some spacing.
  $config->postComment("\n");
  $config->preComment("");

  # Uncomment the following line if you want to use 'ip' instead of 'ifconfig'.
  #$config->networkCommand("ip");

  # turn off synCookie and Martian logging support
  $proc->sysNetIPv4(file => "tcp_syncookies", value => "0");
  $proc->sysNetIPv4Conf(file => "log_martians", value => "0", iface => "all");

  # turn on/off ip forwarding, dynamic IP Addresses, icmp redirects, anti-spoofing blocks and source routing
  $proc->sysNetIPv4(file => "ip_forward", value => "0", module => "Nat", when => "Pre", required => "true");
  $proc->sysNetIPv4(file => "ip_dynaddr", value => "0", module => "Nat");
  $proc->sysNetIPv4Conf(file => "rp_filter", value => "0", module => "Nat", iface => "all");
  $proc->sysNetIPv4Conf(file => "accept_source_route", value => "1", module => "Nat", iface => "all");
  $proc->sysNetIPv4Conf(file => "accept_redirects", value => "1", module => "Nat", iface => "all");
  $proc->sysNetIPv4(file => "icmp_echo_ignore_broadcasts", value => "0", module => "Nat");

  # flush and delete all chains
  $filter->deleteChain(name => "All");
  $mangle->deleteChain(name => "All");
  $nat->deleteChain(name => "All");

  # set chain policies
  $filter->policy(chain => "FORWARD", policy => "DROP");  # If you set this to ACCEPT then your box is wide open!
  $filter->policy(chain => "INPUT", policy => "ACCEPT");
  $filter->policy(chain => "OUTPUT", policy => "ACCEPT");
  $mangle->policy(chain => "OUTPUT", policy => "ACCEPT");
  $mangle->policy(chain => "PREROUTING", policy => "ACCEPT");
  $mangle->policy(chain => "INPUT", policy => "ACCEPT");
  $mangle->policy(chain => "FORWARD", policy => "ACCEPT");
  $mangle->policy(chain => "POSTROUTING", policy => "ACCEPT");
  $nat->policy(chain => "PREROUTING", policy => "ACCEPT");
  $nat->policy(chain => "POSTROUTING", policy => "ACCEPT");
  $nat->policy(chain => "OUTPUT", policy => "ACCEPT");

  # -------------------- Stop Defining your rules -----------------------------
}

# void restartingRules(configObj)
#    Allows the user to specify the installPath to use if they want to
#    change it when we generate the restartfw script.
sub restartingRules
{
  my $self = shift;
  my %args = ( configObj => undef, @_ );
  my $config = $args{configObj};

  # -------------------- Define your rules here -------------------------------

  # if you want/need to change the installPath variable do it here.
  #$config->installPath("/etc/pcx-firewall");  # This is only used in the restartfw script.

  # -------------------- Stop Defining your rules -----------------------------
}

# ******************* User Configuration Ends here ****************************
# *****************************************************************************

sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;

  return $self;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

1;
__END__

=head1 NAME

Rules - The PCXFireWall Rules Module.

=head1 SYNOPSIS

  use PCXFireWall::Rules;
  my $obj = PCXFireWall::Rules->new;

  # define config, nat, filter, mangle, proc, network objects.
  # call the rules system to allow the user to work with them.
  $obj->startingRules(configObj => $config, filterObj => $filter,
                      natObj => $nat, mangleObj => $mangle,
  		      interfaceObj => $interfaces, proc => $proc,
            modulesObj => $modulesObj, networkObj => $networkObj);

  # then we open the file and have each module output its stuff.

=head1 DESCRIPTION

Rules is the Module that Defines the IPTables Rules for the PCXFireWall.

=head1 FUNCTIONS

  scalar new(void)
    Creates a new instance of the PCXFireWall::Rules
    object.

  void startingRules(configObj, interfaceObj, filterObj, natObj,
                     mangleObj, proc, modulesObj, networkObj)
    Defines the rules/actions that need to be done when the firewall is
    starting up.

  void stoppingRules(configObj, interfaceObj, filterObj, natObj,
                     mangleObj, proc, networkObj)
    Defines the rules/actions that need to be done when the firewall is
    shutting down.

  void restartingRules(configObj)
    Allows the user to specify the installPath to use if they want to
    change it when we generate the restartfw script.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin at pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
