#!/usr/bin/perl

#Copyright (c) 2004-2005 Xperience, Inc. http://www.pcxperience.com/
#
# xslt-convert.pl  -  Created by James Pattie, 2004-12-28
# This script will process the specified xml config file
# and attempt to convert it to the latest version.
# The original file is replaced with the converted version,
# if no errors occured and the result is parsable by the
# PCXFireWall::ConfigParser module without any errors.

use strict;
use PCXFireWall::ConfigParser;
use XML::LibXSLT;
use XML::LibXML;

my $VERSION = "1.0";
my $errStr = "(xslt-convert.pl $VERSION) - Error:";
my $module = "";
my $oldFile = "";
my $xslFile = "/usr/share/pcx-firewall-rules/convert.xsl";

my $cfgVersion = "2.2";  # The version of the config file we are converting to.
my $currVersion = "";    # The version of the config file before converting.

if ($#ARGV == 0)
{
  $oldFile = $ARGV[0];
  
  if (! -e $oldFile)
  {
    die "$errStr file = '$oldFile' not found!  $!\n";
  }
  
  # figure out what module we are working with.
  open(FILE, "<$oldFile") or die "$errStr failed to open file = '$oldFile'!  $!\n";
  my @lines = <FILE>;
  my $lines = join('', @lines);
  if ($lines =~ /<firewall module="(VeryTight2?)" moduleVersion="(\d+\.\d+)"/m)
  {
    $module = $1;
    $currVersion = $2;
  }
  else
  {
    die "$errStr Couldn't get module and moduleVersion from:\n$lines";
  }
  close FILE;
}
else
{
  die "Usage: xslt-convert.pl config.xml\n";
}

if ($currVersion == $cfgVersion)
{
  print "$oldFile is already at version $cfgVersion.\n";
  exit 0;
}

my $parser = XML::LibXML->new();
my $xslt = XML::LibXSLT->new();

my $source = undef;
eval { $source = $parser->parse_file($oldFile); };
if ($@)
{
  print("$errStr parse_file('$oldFile') failed!\n");
  print($@);
  exit 1;
}

my $style_doc = undef;
eval { $style_doc = $parser->parse_file($xslFile); };
if ($@)
{
  print("$errStr parse_file('$xslFile') failed!\n");
  print($@);
  exit 1;
}

my $stylesheet = undef;
eval { $stylesheet = $xslt->parse_stylesheet($style_doc); };
if ($@)
{
  print("$errStr parse_stylesheet() failed!\n");
  print($@);
  exit 1;
}

my $results = undef;
eval { $results = $stylesheet->transform($source); };
if ($@)
{
  print("$errStr transform() failed!\n");
  print($@);
  exit 1;
}

# now we need to verify that the PCXFireWall::ConfigParser module likes it.
my $newXML = $stylesheet->output_string($results);

#print $newXML;
#exit 0;

my $confObj = PCXFireWall::ConfigParser->new;

my $dataObj = undef;

eval { $dataObj = $confObj->parse(string => $newXML, module => $module, moduleVersion => $cfgVersion); };
if ($@)
{
  die "$errStr  Eval failed: $@\n";
}

# save the file based upon the xml output from the ConfigParser dataobject, since it will be pretty printed.
my $cleanXML = $dataObj->generateXML;

# backup the file
my $backFile = $oldFile . ".upgrade-$cfgVersion";
`/bin/mv -f $oldFile $backFile`;

open(FILE, ">$oldFile") or die "$errStr Could not create file '$oldFile'! $!\n";
print FILE $cleanXML;
close(FILE);

exit 0;
